Package: PortfolioAnalytics
Type: Package
Title: Portfolio Analysis, Including Numerical Methods for Optimization
        of Portfolios
Authors@R: c(
    person(given=c("Brian","G."),family="Peterson",role=c("cre","aut","cph"),
    email="brian@braverock.com") ,
    person(given="Peter",family="Carl",role=c("aut","cph"),
    email="peter@braverock.com") ,
    person(given="Ross",family="Bennett",role=c("ctb","cph")) ,
    person(given="Kris",family="Boudt",role=c("ctb","cph")) ,
    person(given="Xinran",family="Zhao",role=c("ctb", "cph")),
    person(given="R. Douglas",family="Martin",role="ctb") ,
    person(given="Guy",family="Yollin",role="ctb") ,
    person(given="Hezky",family="Varon",role="ctb") ,
    person(given="Xiaokang",family="Feng",role="ctb") ,
    person(given="Yifu",family="Kang",role="ctb") )
Version: 2.1.2
Date: 2026-04-08
Maintainer: Brian G. Peterson <brian@braverock.com>
Description: Portfolio optimization and analysis routines and graphics.
Depends: R (>= 4.0.0), zoo, xts (>= 0.10-1), foreach,
        PerformanceAnalytics (>= 1.5.1)
Suggests: quantmod, DEoptim (>= 2.2.1), iterators, doParallel, doMC,
        fGarch, Rglpk, quadprog, ROI (>= 0.1.0), ROI.plugin.glpk (>=
        0.0.2), ROI.plugin.quadprog (>= 0.0.2), corpcor, testthat,
        nloptr (>= 1.0.0), MASS, robustbase, osqp, CVXR, data.table,
        knitr, rmarkdown, GSE, RobStatTM, PCRA, R.rsp, RPESE, TTR,
        Matrix
VignetteBuilder: R.rsp
Imports: methods, GenSA, ROI.plugin.symphony, mco, pso
License: GPL-3
URL: https://github.com/braverock/PortfolioAnalytics
Copyright: (c) 2004-2026
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-04-09 19:08:52 UTC; brian
Author: Brian G. Peterson [cre, aut, cph],
  Peter Carl [aut, cph],
  Ross Bennett [ctb, cph],
  Kris Boudt [ctb, cph],
  Xinran Zhao [ctb, cph],
  R. Douglas Martin [ctb],
  Guy Yollin [ctb],
  Hezky Varon [ctb],
  Xiaokang Feng [ctb],
  Yifu Kang [ctb]
Repository: CRAN
Date/Publication: 2026-04-10 05:10:17 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 00:38:03 UTC; unix
Archs: PortfolioAnalytics.so.dSYM
