# Generated by roxygen2: do not edit by hand

S3method(chart.EF.Weights,efficient.frontier)
S3method(chart.EF.Weights,optimize.portfolio)
S3method(chart.EfficientFrontier,efficient.frontier)
S3method(chart.EfficientFrontier,optimize.portfolio)
S3method(chart.EfficientFrontier,optimize.portfolio.CVXR)
S3method(chart.EfficientFrontier,optimize.portfolio.ROI)
S3method(chart.RiskBudget,opt.list)
S3method(chart.RiskBudget,optimize.portfolio)
S3method(chart.RiskBudget,optimize.portfolio.rebalancing)
S3method(chart.RiskReward,opt.list)
S3method(chart.RiskReward,optimize.portfolio.DEoptim)
S3method(chart.RiskReward,optimize.portfolio.GenSA)
S3method(chart.RiskReward,optimize.portfolio.ROI)
S3method(chart.RiskReward,optimize.portfolio.pso)
S3method(chart.RiskReward,optimize.portfolio.random)
S3method(chart.Weights,opt.list)
S3method(chart.Weights,optimize.portfolio.DEoptim)
S3method(chart.Weights,optimize.portfolio.GenSA)
S3method(chart.Weights,optimize.portfolio.ROI)
S3method(chart.Weights,optimize.portfolio.pso)
S3method(chart.Weights,optimize.portfolio.random)
S3method(chart.Weights,optimize.portfolio.rebalancing)
S3method(extractObjectiveMeasures,opt.list)
S3method(extractObjectiveMeasures,opt.rebal.list)
S3method(extractObjectiveMeasures,optimize.portfolio)
S3method(extractObjectiveMeasures,optimize.portfolio.rebalancing)
S3method(extractObjectiveMeasures,summary.optimize.portfolio.rebalancing)
S3method(extractStats,opt.list)
S3method(extractStats,opt.rebal.list)
S3method(extractStats,optimize.portfolio.CVXR)
S3method(extractStats,optimize.portfolio.DEoptim)
S3method(extractStats,optimize.portfolio.GenSA)
S3method(extractStats,optimize.portfolio.ROI)
S3method(extractStats,optimize.portfolio.eqwt)
S3method(extractStats,optimize.portfolio.invol)
S3method(extractStats,optimize.portfolio.parallel)
S3method(extractStats,optimize.portfolio.pso)
S3method(extractStats,optimize.portfolio.random)
S3method(extractStats,optimize.portfolio.rebalancing)
S3method(extractWeights,opt.list)
S3method(extractWeights,opt.rebal.list)
S3method(extractWeights,optimize.portfolio)
S3method(extractWeights,optimize.portfolio.rebalancing)
S3method(extractWeights,summary.optimize.portfolio.rebalancing)
S3method(plot,optimize.portfolio)
S3method(plot,optimize.portfolio.DEoptim)
S3method(plot,optimize.portfolio.GenSA)
S3method(plot,optimize.portfolio.ROI)
S3method(plot,optimize.portfolio.pso)
S3method(plot,optimize.portfolio.random)
S3method(print,constraint)
S3method(print,efficient.frontier)
S3method(print,opt.list)
S3method(print,opt.rebal.list)
S3method(print,optimize.portfolio.CVXR)
S3method(print,optimize.portfolio.DEoptim)
S3method(print,optimize.portfolio.GenSA)
S3method(print,optimize.portfolio.ROI)
S3method(print,optimize.portfolio.parallel)
S3method(print,optimize.portfolio.pso)
S3method(print,optimize.portfolio.random)
S3method(print,optimize.portfolio.rebalancing)
S3method(print,portfolio)
S3method(print,portfolio.list)
S3method(print,regime.portfolios)
S3method(print,summary.optimize.portfolio)
S3method(print,summary.optimize.portfolio.rebalancing)
S3method(summary,efficient.frontier)
S3method(summary,optimize.portfolio)
S3method(summary,optimize.portfolio.parallel)
S3method(summary,optimize.portfolio.rebalancing)
S3method(summary,portfolio)
S3method(update,constraint)
export(CCCgarch.MM)
export(EntropyProg)
export(HHI)
export(MycovRobMcd)
export(MycovRobTSGS)
export(ac.ranking)
export(add.constraint)
export(add.objective)
export(add.objective_v1)
export(add.objective_v2)
export(add.sub.portfolio)
export(applyFUN)
export(backtest.plot)
export(black.litterman)
export(box_constraint)
export(center)
export(centroid.buckets)
export(centroid.complete.mc)
export(centroid.sectors)
export(centroid.sign)
export(chart.Concentration)
export(chart.EF.Weights)
export(chart.EfficientFrontier)
export(chart.EfficientFrontierCompare)
export(chart.EfficientFrontierOverlay)
export(chart.GroupWeights)
export(chart.RiskBudget)
export(chart.RiskReward)
export(chart.Weights)
export(combine.optimizations)
export(combine.portfolios)
export(constrained_objective)
export(constrained_objective_v1)
export(constrained_objective_v2)
export(constraint)
export(constraint_ROI)
export(constraint_v1)
export(constraint_v2)
export(create.EfficientFrontier)
export(custom.covRob.MM)
export(custom.covRob.Mcd)
export(custom.covRob.Rocke)
export(custom.covRob.TSGS)
export(diversification)
export(diversification_constraint)
export(equal.weight)
export(extractCokurtosis)
export(extractCoskewness)
export(extractCovariance)
export(extractEfficientFrontier)
export(extractGroups)
export(extractObjectiveMeasures)
export(extractStats)
export(extractWeights)
export(extract_risk)
export(factor_exposure_constraint)
export(fn_map)
export(generatesequence)
export(group_constraint)
export(insert_objectives)
export(inverse.volatility.weight)
export(is.constraint)
export(is.objective)
export(is.portfolio)
export(leverage_exposure_constraint)
export(meancsm.efficient.frontier)
export(meaneqs.efficient.frontier)
export(meanetl.efficient.frontier)
export(meanrisk.efficient.frontier)
export(meanvar.efficient.frontier)
export(meucci.moments)
export(meucci.ranking)
export(minmax_objective)
export(mult.portfolio.spec)
export(objective)
export(opt.outputMvo)
export(optimize.portfolio)
export(optimize.portfolio.parallel)
export(optimize.portfolio.rebalancing)
export(optimize.portfolio.rebalancing_v1)
export(optimize.portfolio_v1)
export(optimize.portfolio_v2)
export(plotFrontiers)
export(portfolio.spec)
export(portfolio_risk_objective)
export(pos_limit_fail)
export(position_limit_constraint)
export(quadratic_utility_objective)
export(random_portfolios)
export(random_portfolios_v1)
export(random_portfolios_v2)
export(random_walk_portfolios)
export(randomize_portfolio)
export(randomize_portfolio_v1)
export(randomize_portfolio_v2)
export(regime.portfolios)
export(return_constraint)
export(return_objective)
export(risk_budget_objective)
export(rp_grid)
export(rp_sample)
export(rp_simplex)
export(rp_transform)
export(scatterFUN)
export(set.portfolio.moments)
export(statistical.factor.model)
export(trailingFUN)
export(transaction_cost_constraint)
export(turnover)
export(turnover_constraint)
export(turnover_objective)
export(update_constraint_v1tov2)
export(var.portfolio)
export(weight_concentration_objective)
export(weight_sum_constraint)
import(GenSA)
import(PerformanceAnalytics)
import(ROI.plugin.symphony)
import(foreach)
import(mco)
import(xts)
import(zoo)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRamp)
importFrom(grDevices,heat.colors)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(methods,hasArg)
importFrom(pso,lines)
importFrom(pso,plot)
importFrom(pso,points)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,end)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,optimize)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(utils,head)
importFrom(utils,sessionInfo)
useDynLib("PortfolioAnalytics")
